      program hbxca
!
! Given a list of contact distances rco(th1,th2), compute the 
! excluded area Axc(th12), for th12 (=th2-th1) in [0 .. 2*pi]. 
!
! USAGE:
! - run spell2d.f with parameter file spell.par; (set "test=f")
! - copy file spell.par --> hbxca.par
! - copy file spell.rco --> hbxca.rco
! - run hbxca.f
!
! Integration: Simpson 4-2
!
      implicit double precision (a-h,o-z)
      dimension xlen(2),rc0(0:500,0:500),rcr(0:500,0:500)
     1 ,acc(0:100,0:100),acs(0:100,0:100),asc(0:100,0:100)
     2 ,ass(0:100,0:100),theta1(0:500),theta2(0:500)
      common aa,cc,z1,th1,th2
      character(70) headline,ident,sdummy 
      character(35) text  
      logical test 
!
      pi=4.d0*datan(1.d0)
!
      open(unit=4,file='hbxca.par',form='formatted')
      open(unit=5,file='hbxca.rco',form='formatted') 
      open(unit=7,file='hbxca.out',form='formatted')
      open(unit=8,file='hbxca.dat',form='formatted')
!     ~.par ... (in)  parameters of the hard particle contact computation
!     ~.rco ... (in)  table of contact distances rco(th1,th2)
!     ~.out ... (out) output file
!     ~.dat ... (out) table of excluded volumes Axc(th12)
!
      write(7,700)
700   format(1x,'Excluded Area of Hard Objects in 2D:')
!
      read(4,'(a/a)') sdummy,ident
      write(*,101) ident
101   format(1x/72("-")//
     1  "     Excluded Volume of Hard Objects in 2D    "/
     1  "       Program: hbxca - Hard Body Excluded Area 12.10 "/
     1  "       Author: F. J. Vesely  | Oct-12                 "//
     1  72("-")//1x,a)
      read(4,'(a/3f10.5)') sdummy,aa,cc,z1
      write(*,102) aa,cc,z1
102   format(1x/
     1  1x,"Particle shape:   ", 
     1  1x," aa   =",f6.2,"      cc  =",f6.2
     1  ,"        z1   =",f6.2 
     1  )
      read(4,'(a/3i10,l5)') sdummy,nth,nt1,nt2,test
      write(*,105) nth,nt1,nt2,test
105   format(1x,"Computational :",
     1  4x,"nth   =",i6,"     nt1  =",i6,"       nt2   =",i6
     1  /19x,"test   =",l6 
     1  )
! 
! If test=true then the table rco(ith1,ith2) has no entries,
! and we cannot proceed. 
!
      if (test) then
        print *,'no usable table of contact distances - stopping'
        stop 33
      endif
!
!  Geometry of the DiscoEllipse DE:
      asq=aa*aa; csq=cc*cc; z1sq=z1*z1;
      x1=dsqrt(asq*(1.d0-z1sq/csq));
      z0=z1*(1.d0-asq/csq); z0sq=z0*z0;
      rcsq=asq*(1.d0-(z1sq/csq)*(1.d0-asq/csq));
      rc=dsqrt(rcsq); ! cap radius; z0+rc is half length of SE
!    Private area:
      xld=z0+rc; xwd=aa; hd=z1-z0;
      rt=dsqrt(csq-z1sq)
      term1=0.5d0*pi; if(z1sq.lt.csq) term1=datan(z1/rt);
      apde1=(2.d0*aa/cc)*(z1*rt+csq*term1) 
      term2=0.d0; if(hd.lt.rc) term2=dacos(hd/rc);
      apde2=rcsq*term2         
      apde=apde1+apde2              ! private area of DiscoEllipse
!
      write(*,299) 4.d0*apde,apde
299   format(/1x,80('=')
     1 //1x,'EXCLUDED VOLUME:'
     1 //1x,'Keep two DiscoEllipses at a constant relative '
     1 ,'angle th12 and rotate'/1x,'their center-center vector full'
     1,' circle, phi=0..2*pi;'
     1 /1x,'integrate the square of the contact distance rco over '
     1 ,'all phi:'//1x,'Axc(th12) = (1/2) int dphi (rco^2)'
     1 //1x,'DiscoEllipses have a well-known "self" '
     1 ,'area A0, and the excluded area'/1x,'for two parallel'
     1 ,' DEs is just Axc=4*A0. For a general relative'
     1 /1x,'orientation th12 we have no closed expression'
     1 ,1x,'(yet) for Axc, but we'/1x,'can calculate it from'
     1 ,1x,'the table rco(th1,th2) by Simpson integration.'
     1 //1x,'However, we may define "effective" Ellipses (EE) and'
     1 ,1x,'DiscoRectangles (DR)'/1x,'by demanding they have the'
     1 ,1x,'same aspect ratio and area as the DE. For these'/1x
     1 ,'objects we can indeed calculate theoretical values of'
     1,1x,'Axc(th12).'
     1 //1x,'DiscoEllipses, th12=0:'
     1 ,7x,'Axc(par) = ',f12.6,' = 4*A0 = 4*(',f12.6,')'
     1 //1x,'A full list of Axc(th12) is in "hbxc.dat" '//)
!
! Now read in table of rco(th1,th2):
      dth=2.d0*pi/dfloat(nth);
      do 344 ith1=0,nth
        do 345 ith2=0,nth
          rc0(ith1,ith2)=(0.d0,0.d0)
          rcr(ith1,ith2)=(0.d0,0.d0)
          read(5,555) text,it1,th1,it2,th2,rco,rgl,utest
 !         write(*,555) text,it1,th1,it2,th2,rco,rgl,utest
555       format(a,i5,f10.5,i5,f10.5,3e13.5)
          theta1(ith1)=th1*pi
          theta2(ith2)=th2*pi
          rc0(ith1,ith2)=rco
345     continue
344   continue
!-------------------------------------
!  For a given th2 and th1 compute the excluded volume Axc(th2-th1):
      nt1=0; ! first particle orientation 
      do 11 nphi=0,nth/2
        nt12=iabs(nphi-nt1);
        th12=nt12*dth ! relative angle th2-th1
        call arex(rc0,dth,nth,nt1,nt12,arxc,axee,axdr)
        write(8,304) th12/pi,arxc,axee,axdr
304     format(1x,'th12/pi,Axc(DE-EE-DR)=',4f15.6)
        idth=nth/10
        if((nt12/idth)*idth.eq.nt12) then
          write(*,304) th12/pi,arxc,axee,axdr
        endif
11    continue
!
!-------------------------------------      
!
      write(*,305)
305   format(//1x,80('='))

      stop
      end

!----------------------------------------------------
!
      subroutine arex(rc0,dth,nth,nt1,nt12,arxc,axee,axdr)
!
! Compute excluded volume by Simpson integration:
! Let a table of rco(th1,th2) be given, where th1-2 are the individual 
! orientations, measured from vertical to the right, and where particle 2 
! is assumed to be positioned on the vertical. 
! Let us define Rco(th1,th2;phi) as the contact distance when 2 is not
! on the vertical but on a vector with orientation phi measured as above.
! Then Rco(th1,th2,phi) = rco(th1-phi,th2-phi).
! The excluded area is then given by 
! Axc=(1/2) int_[-pi]^[+pi] dphi rco^2(-phi,th2-phi)
! (where we have put th1=0). 
! We can do a Simpson integration if we set 
! phi_i=-pi+i*(2pi/n), i=0..n (n even, no. of terms odd.)
! th2 (=th12) = nt12*(2pi/n) :
!
! Axc=(1/2) Sum_[i=0]^[n] (2pi/n) [R(i)]^2  where by symmetry
! R(i)=rco(n-i,nt12-i) for i<nt12 and 
! R(i)=rco(n-i,n-(i-nt12)) for i>=nt12
!
      implicit double precision (a-h,o-z)
      dimension rc0(0:500,0:500)
      logical test
      common aa,cc,z1,th1,th2
!
      pi=4.d0*datan(1.0d0); pih=0.5d0*pi 
!
!  Geometry of the DiscoEllipse DE:
      asq=aa*aa; csq=cc*cc; z1sq=z1*z1;
      x1=dsqrt(asq*(1.d0-z1sq/csq));
      z0=z1*(1.d0-asq/csq); z0sq=z0*z0;
      rcsq=asq*(1.d0-(z1sq/csq)*(1.d0-asq/csq));
      rc=dsqrt(rcsq); ! cap radius; z0+rc is half length of SE
!    Private area:
      xld=z0+rc; xwd=aa; hd=z1-z0;
      rt=dsqrt(csq-z1sq)
      term1=0.5d0*pi; if(z1sq.lt.csq) term1=datan(z1/rt);
      apde1=(2.d0*aa/cc)*(z1*rt+csq*term1) 
      term2=0.d0; if(hd.lt.rc) term2=dacos(hd/rc);
      apde2=rcsq*term2         
      apde=apde1+apde2              ! private area of DiscoEllipse
!
      co1=dth*1.d0/3.d0
      co2=dth*2.d0/3.d0
      co4=dth*4.d0/3.d0  ! Simpson coefficients
!
      nphi=nt12;
      th12=nphi*dth ! relative angle th2-th1
      arxc=0.d0
      do 111 i=0,nth
        co=co4
        if(2*(i/2).eq.i) co=co2
        if(i.eq.nth.or.i.eq.0) co=co1
        if((nphi-i).ge.0) then
          if((nth-i).lt.0) stop 65
          rri=rc0(nth-i,nphi-i)
        else
          rri=rc0(nth-i,nth-(i-nphi))
          if((nth-(i-nphi)).lt.0) stop 67
        endif
        arxc=arxc+co*rri*rri
111   continue
      arxc=0.5d0*arxc  ! numerical estimate of Axc
!
!  Compare to theoretical Axc(th2-th1) of ellipse and discorectangle:
!
! (1) Representative Ellipse RE: has the same aspect ratio and area as DE
      cre=dsqrt(apde*(z0+rc)/aa/pi); ! long axis of the repr. ellipse 
      are=aa*cre/(z0+rc);            ! short axis 
      ae=cre/are    ! aspect ratio
      apee=cre*are*pi;               ! private area of Repr.Ellipse  
!   Compute integral K(th12):
      call schlacken(ae,th12,xkk)
      axee=0.5d0*pi*cre*are*(1.d0+xkk)
      axee=4.d0*axee  ! because we use aa,cc for the HALF axes
!
! (2) DiscoRectangle:
      anen=0.25d0*pi+(z0+rc-aa)/aa
      drr=dsqrt(apde/anen)      ! width of Repr. DiscoRectangle
      xlr=drr*(z0+rc-aa)/aa     ! length of Rectangle(!)
      ar=xlr/drr      ! aspect ratio of Rectangle(!)
      apdr=drr*drr*(0.25d0*pi+ar)   ! private area of DiscoRectangle
      axdr=drr*drr*(pi+4.d0*ar+ar*ar*dabs(dsin(th12))) 
!
      return
      end
!
!------------------------------------------------------
!
      subroutine schlacken(ae,th,xkk)
!
! Compute Schlacken's integral by Simpson quadratu
! vxc=(pi/2)*cc*aa*(1+K(w1-w2))
!
! where (Schlacken)
! K(w)=(1/2pi)*int_0^2pi dt 
! [(cos(w)*cos(t) - ae* sin(w)*sin(t))^2 + 
! + (sin(w)*cos(t)/ae +cos(w)*sin(t) )^2 ]^(1/2)
! and ae=cc/aa
!
      implicit double precision (a-h,o-z)
!
      pi=4.d0*datan(1.0d0); pih=0.5d0*pi 
!
      cth=dcos(th)
      sth=dsin(th)
      ns=201 
      nn=ns-1
      dt=2.d0*pi/(ns-1)
      co1=dt*1.d0/3.d0
      co2=dt*2.d0/3.d0
      co4=dt*4.d0/3.d0  ! Simpson coefficients
      xkk=0.d0
      do 111 i=0,nn
        co=co4
        if(2*(i/2).eq.i) co=co2
        if(i.eq.nn.or.i.eq.0) co=co1
        t=i*dt
        bra1=(cth*dcos(t)-ae*sth*dsin(t))
        term1=bra1*bra1
        bra2=(sth*dcos(t)/ae+cth*dsin(t))
        term2=bra2*bra2
        xkk=xkk+co*dsqrt(term1+term2)
111   continue
      xkk=0.5d0*xkk/pi
!
      return
      end
!
!-----------------------------------------------------------------
